#include "stdafx.h"
#include "AIO167.h"
#include "AIO16A.h"

int AIO16CardNum = 0;
unsigned char jumpers;
unsigned char adrange;
unsigned char dacBrange;
unsigned char dacArange;

// sbitd takes data and bit, then returns 0x81 or 0x01 based on data[bit]
unsigned char sbitd(unsigned int data,unsigned char bit)
{
  return (data & (1<<bit))?0x81:0x01;
}


double DAC(unsigned char DACnum,double DACV)
{
  DACnum &= 1;

  int count = (int) (DACV/((DACnum?dacBrange:dacArange)?5.0L:10.0L) * 0x1000);
  if (count > 0xFFF)
    count = 0xFFF;
  else if (count < 0)
    count = 0;

  RelOutPort(AIO16CardNum, 0xC + (DACnum*2), count);
  // outport(BASE + 0x08, 0x8000);

  return double(count)/4096.0L*((DACnum?dacArange:dacBrange)?5.0L:10.0L);  // actual V
}


void EEPROM_Enable(void)
{
  RelOutPortB(AIO16CardNum, 0x18,0x80);  // start process
  RelOutPortB(AIO16CardNum, 0x18,0x81);  // start bit
  RelOutPortB(AIO16CardNum, 0x18,0x01);  // Opcode bit 0 (00 = enable/disable)
  RelOutPortB(AIO16CardNum, 0x18,0x01);  // Opcode bit 1  ""
  RelOutPortB(AIO16CardNum, 0x18,0x81);  // a5 (must be 1 to enable)
  RelOutPortB(AIO16CardNum, 0x18,0x81);  // a4 (must be 1 to enable)
  RelOutPortB(AIO16CardNum, 0x18,0x01);  // a3 (don't care because enable/disable)
  RelOutPortB(AIO16CardNum, 0x18,0x01);  // a2 (don't care because enable/disable)
  RelOutPortB(AIO16CardNum, 0x18,0x01);  // a1 (don't care because enable/disable)
  RelOutPortB(AIO16CardNum, 0x18,0x01);  // a0 (don't care because enable/disable)
  RelOutPortB(AIO16CardNum, 0x18,0x00);  // end trans
}

void EEPROM_Disable(void)
{
  RelOutPortB(AIO16CardNum, 0x18,0x80);  // start process
  RelOutPortB(AIO16CardNum, 0x18,0x81);  // start bit
  RelOutPortB(AIO16CardNum, 0x18,0x01);  // Opcode bit 0 (00 = enable/disable)
  RelOutPortB(AIO16CardNum, 0x18,0x01);  // Opcode bit 1  ""
  RelOutPortB(AIO16CardNum, 0x18,0x01);  // a5 (must be 0 to disable)
  RelOutPortB(AIO16CardNum, 0x18,0x01);  // a4 (must be 0 to disable)
  RelOutPortB(AIO16CardNum, 0x18,0x01);  // a3 (don't care because enable/disable)
  RelOutPortB(AIO16CardNum, 0x18,0x01);  // a2 (don't care because enable/disable)
  RelOutPortB(AIO16CardNum, 0x18,0x01);  // a1 (don't care because enable/disable)
  RelOutPortB(AIO16CardNum, 0x18,0x01);  // a0 (don't care because enable/disable)
  RelOutPortB(AIO16CardNum, 0x18,0x00);  // end trans
}

void EEPROM_Write(unsigned int addr,unsigned int data)
{
  RelOutPortB(AIO16CardNum, 0x18,0x80);  // start process
  RelOutPortB(AIO16CardNum, 0x18,0x81);  // Start Bit
  RelOutPortB(AIO16CardNum, 0x18,0x01);  // Opcode Bit 0 (01=write)
  RelOutPortB(AIO16CardNum, 0x18,0x81);  // Opcode Bit 1  ""
  RelOutPortB(AIO16CardNum, 0x18,sbitd(addr,5));  // a5 (address to write to)
  RelOutPortB(AIO16CardNum, 0x18,sbitd(addr,4));  // a4
  RelOutPortB(AIO16CardNum, 0x18,sbitd(addr,3));  // a3
  RelOutPortB(AIO16CardNum, 0x18,sbitd(addr,2));  // a2
  RelOutPortB(AIO16CardNum, 0x18,sbitd(addr,1));  // a1
  RelOutPortB(AIO16CardNum, 0x18,sbitd(addr,0));  // a0
  RelOutPortB(AIO16CardNum, 0x18,sbitd(data,15));  // d15
  RelOutPortB(AIO16CardNum, 0x18,sbitd(data,14));  // d14
  RelOutPortB(AIO16CardNum, 0x18,sbitd(data,13));  // d13
  RelOutPortB(AIO16CardNum, 0x18,sbitd(data,12));  // d12
  RelOutPortB(AIO16CardNum, 0x18,sbitd(data,11));  // d11
  RelOutPortB(AIO16CardNum, 0x18,sbitd(data,10));  // d10
  RelOutPortB(AIO16CardNum, 0x18,sbitd(data,9));  // d9
  RelOutPortB(AIO16CardNum, 0x18,sbitd(data,8));  // d8
  RelOutPortB(AIO16CardNum, 0x18,sbitd(data,7));  // d7
  RelOutPortB(AIO16CardNum, 0x18,sbitd(data,6));  // d6
  RelOutPortB(AIO16CardNum, 0x18,sbitd(data,5));  // d5
  RelOutPortB(AIO16CardNum, 0x18,sbitd(data,4));  // d4
  RelOutPortB(AIO16CardNum, 0x18,sbitd(data,3));  // d3
  RelOutPortB(AIO16CardNum, 0x18,sbitd(data,2));  // d2
  RelOutPortB(AIO16CardNum, 0x18,sbitd(data,1));  // d1
  RelOutPortB(AIO16CardNum, 0x18,sbitd(data,0));  // d0
  RelOutPortB(AIO16CardNum, 0x18,0x00);  // end trans

  Sleep(20);
}

unsigned int EEPROM_Read(unsigned int addr)
{
  unsigned int data=0;
  RelOutPortB(AIO16CardNum, 0x18,0x80);  // start process
  RelOutPortB(AIO16CardNum, 0x18,0x81);  // Start Bit
  RelOutPortB(AIO16CardNum, 0x18,0x81);  // Opcode Bit 0 (10=Read)
  RelOutPortB(AIO16CardNum, 0x18,0x01);  // Opcode Bit 1  ""
  RelOutPortB(AIO16CardNum, 0x18,sbitd(addr,5));  // a5 (address to write to)
  RelOutPortB(AIO16CardNum, 0x18,sbitd(addr,4));  // a4
  RelOutPortB(AIO16CardNum, 0x18,sbitd(addr,3));  // a3
  RelOutPortB(AIO16CardNum, 0x18,sbitd(addr,2));  // a2
  RelOutPortB(AIO16CardNum, 0x18,sbitd(addr,1));  // a1
  RelOutPortB(AIO16CardNum, 0x18,sbitd(addr,0));  // a0
  data=(data<<1) | ((RelInPortB(AIO16CardNum, 0x18) & 0x80)?1:0);  // d15
  data=(data<<1) | ((RelInPortB(AIO16CardNum, 0x18) & 0x80)?1:0);  // d14
  data=(data<<1) | ((RelInPortB(AIO16CardNum, 0x18) & 0x80)?1:0);  // d13
  data=(data<<1) | ((RelInPortB(AIO16CardNum, 0x18) & 0x80)?1:0);  // d12
  data=(data<<1) | ((RelInPortB(AIO16CardNum, 0x18) & 0x80)?1:0);  // d11
  data=(data<<1) | ((RelInPortB(AIO16CardNum, 0x18) & 0x80)?1:0);  // d10
  data=(data<<1) | ((RelInPortB(AIO16CardNum, 0x18) & 0x80)?1:0);  // d9
  data=(data<<1) | ((RelInPortB(AIO16CardNum, 0x18) & 0x80)?1:0);  // d8
  data=(data<<1) | ((RelInPortB(AIO16CardNum, 0x18) & 0x80)?1:0);  // d7
  data=(data<<1) | ((RelInPortB(AIO16CardNum, 0x18) & 0x80)?1:0);  // d6
  data=(data<<1) | ((RelInPortB(AIO16CardNum, 0x18) & 0x80)?1:0);  // d5
  data=(data<<1) | ((RelInPortB(AIO16CardNum, 0x18) & 0x80)?1:0);  // d4
  data=(data<<1) | ((RelInPortB(AIO16CardNum, 0x18) & 0x80)?1:0);  // d3
  data=(data<<1) | ((RelInPortB(AIO16CardNum, 0x18) & 0x80)?1:0);  // d2
  data=(data<<1) | ((RelInPortB(AIO16CardNum, 0x18) & 0x80)?1:0);  // d1
  data=(data<<1) | ((RelInPortB(AIO16CardNum, 0x18) & 0x80)?1:0);  // d0
  RelOutPortB(AIO16CardNum, 0x18,0x00);  // end com
  return data;
}

void CAL_ADC(unsigned char M,unsigned char B)
{ // write cal to A/D, offset first, then slope
  RelOutPortB(AIO16CardNum, 0x19,0x80);  // start process
  RelOutPortB(AIO16CardNum, 0x19,0x01);  // a1 pot 00 is offset
  RelOutPortB(AIO16CardNum, 0x19,0x01);  // a0 ""
  RelOutPortB(AIO16CardNum, 0x19,sbitd(B,7));  // data bit 7 of offset
  RelOutPortB(AIO16CardNum, 0x19,sbitd(B,6));  // data bit 6 of offset
  RelOutPortB(AIO16CardNum, 0x19,sbitd(B,5));  // data bit 5 of offset
  RelOutPortB(AIO16CardNum, 0x19,sbitd(B,4));  // data bit 4 of offset
  RelOutPortB(AIO16CardNum, 0x19,sbitd(B,3));  // data bit 3 of offset
  RelOutPortB(AIO16CardNum, 0x19,sbitd(B,2));  // data bit 2 of offset
  RelOutPortB(AIO16CardNum, 0x19,sbitd(B,1));  // data bit 1 of offset
  RelOutPortB(AIO16CardNum, 0x19,sbitd(B,0));  // data bit 0 of offset
  RelOutPortB(AIO16CardNum, 0x19,0x00);  // end trans
  Sleep(1);
  // write slope now
  RelOutPortB(AIO16CardNum, 0x19,0x80);  // start process
  RelOutPortB(AIO16CardNum, 0x19,0x01);  // a1 pot 01 is slope of A/D
  RelOutPortB(AIO16CardNum, 0x19,0x81);  // a0 pot 01 is slope of A/D
  RelOutPortB(AIO16CardNum, 0x19,sbitd(M,7));  // data bit 7 of slope
  RelOutPortB(AIO16CardNum, 0x19,sbitd(M,6));  // data bit 6 of slope
  RelOutPortB(AIO16CardNum, 0x19,sbitd(M,5));  // data bit 5 of slope
  RelOutPortB(AIO16CardNum, 0x19,sbitd(M,4));  // data bit 4 of slope
  RelOutPortB(AIO16CardNum, 0x19,sbitd(M,3));  // data bit 3 of slope
  RelOutPortB(AIO16CardNum, 0x19,sbitd(M,2));  // data bit 2 of slope
  RelOutPortB(AIO16CardNum, 0x19,sbitd(M,1));  // data bit 1 of slope
  RelOutPortB(AIO16CardNum, 0x19,sbitd(M,0));  // data bit 0 of slope
  RelOutPortB(AIO16CardNum, 0x19,0x00);  // end trans
}

void CAL_DAC(unsigned char DACNum,unsigned char M)
{
  // PCICARDs use +1A instead of +19, and the A1 bit needs to be cleared
  // we can read BASE+0x1F to determine if its the PCI card or not.  PCI==0
  unsigned PCICARD=(RelInPortB(AIO16CardNum, 0x1F)==0x00) ? 1 : 0;

  RelOutPortB(AIO16CardNum, 0x19+PCICARD,0x80);  // start process
  RelOutPortB(AIO16CardNum, 0x19+PCICARD,PCICARD?0x01:0x81 );
  RelOutPortB(AIO16CardNum, 0x19+PCICARD,sbitd(DACNum,0));  // a0 pot 11 is slope of DAC1
  RelOutPortB(AIO16CardNum, 0x19+PCICARD,sbitd(M,7));  // data bit 7 of slope
  RelOutPortB(AIO16CardNum, 0x19+PCICARD,sbitd(M,6));  // data bit 6 of slope
  RelOutPortB(AIO16CardNum, 0x19+PCICARD,sbitd(M,5));  // data bit 5 of slope
  RelOutPortB(AIO16CardNum, 0x19+PCICARD,sbitd(M,4));  // data bit 4 of slope
  RelOutPortB(AIO16CardNum, 0x19+PCICARD,sbitd(M,3));  // data bit 3 of slope
  RelOutPortB(AIO16CardNum, 0x19+PCICARD,sbitd(M,2));  // data bit 2 of slope
  RelOutPortB(AIO16CardNum, 0x19+PCICARD,sbitd(M,1));  // data bit 1 of slope
  RelOutPortB(AIO16CardNum, 0x19+PCICARD,sbitd(M,0));  // data bit 0 of slope
  RelOutPortB(AIO16CardNum, 0x19+PCICARD,0x00);  // end trans
}

void CALWrite(unsigned zAM,unsigned zAB,unsigned zDAM,unsigned zDBM)
{
// EEPROM_Enable();
 EEPROM_Write((adrange+0),zAB);
 EEPROM_Write((adrange+8),zAM);
 EEPROM_Write((dacArange+0x10),zDAM);
 EEPROM_Write((dacBrange+0x12),zDBM);
// EEPROM_Disable();
}

void doCAL(void)
{
  unsigned char zDAM, zDBM, zAM, zAB;

  zAB=EEPROM_Read((adrange+0))&0xff;  //byte 0-7 in EEPROM holds "B" for ADC
  zAM=EEPROM_Read((adrange+8))&0xff;  //byte 8-f in EEPROM holds "M" for ADC
  zDAM=EEPROM_Read((dacArange+0x10))&0xff; // byte x10,x11 in EEPROM holds "M" for DAC0
  zDBM=EEPROM_Read((dacBrange+0x12))&0xff; // byte x12,x13 in EEPROM holds "M" for DAC1
  CAL_DAC(0,zDAM);  // write the calibration value from EEPROM to CAL Pots
  CAL_DAC(1,zDBM);  //  ""
  CAL_ADC(zAM,zAB);  //  ""
}

void init(void)
{
  jumpers = UINT8(RelInPortB(AIO16CardNum, 0x12));  // jumper configuration readback
  adrange=jumpers & 0x07;  // d2,d1,d0 are 5/10, uni/bip, 16/8
  dacBrange=(jumpers>>3) & 0x01;  // d4,d3 are 5/10 for daca,b
  dacArange=(jumpers>>4) & 0x01;
  //   CAL();
}

